include <library.scad>

// Depth of detail.
emboss_height = unit/16;
// Gap around edges where panels will possibly meet.
tolerance = 0.1;

module plate(x_units, y_units, prepare_to_emboss=false) difference() {
	assert(x_units > 0);
	assert(y_units > 0);
	
	height = prepare_to_emboss ? unit/2-emboss_height : unit/2;

	translate([tolerance-grid/2, tolerance-grid/2, -unit/2]) cube([grid*x_units-tolerance*2, grid*y_units-tolerance*2, height]);

	// Cut v-slots.
	for (x = [0:x_units-1]) for (y = [0:y_units-1]) {
		translate([x*grid, y*grid, 0]) vslots_z_negative();
	}

	// Cut bevels.
	for (x = [0:x_units-1]) translate([x*grid, 0, 0]) {
		rotate(90) rotate(90, [1, 0, 0]) bevel();
		translate([0, grid*(y_units-1), 0]) rotate(270) rotate(90, [1, 0, 0]) bevel();
	}
	for (y = [0:y_units-1]) translate([0, y*grid, 0]) {
		rotate(90, [1, 0, 0]) bevel();
		translate([grid*(x_units-1), 0, 0]) rotate(180) rotate(90, [1, 0, 0]) bevel();
	}
}

module plate_1x1_slatted() {
	slat_count = 3;
	slat_height = grid/slat_count/2;

	plate(1, 1, true);

	for (y = [0:slat_count-1]) translate([-grid/2, -grid/2+slat_height*2*y, -emboss_height]) cube([grid, slat_height, emboss_height]);
}

module plate_1x1_piped_0() {
	plate(1, 1, true);
	
	intersection() {
		translate([0, 0, -emboss_height]) union() {
			pipe([
				[-grid/4, -grid/2],
				[-grid/4, grid/2],
			]);
			pipe([
				[0, -grid/2],
				[0, -grid*0.1],
				[grid/4, grid*0.1],
				[grid/4, grid/2],
			]);
			pipe([
				[0, grid*0.3],
				[0, grid/2],
			]);
			pipe([
				[grid/4, -grid/2],
				[grid/4, -grid*0.2],
				[grid/2, 0],
			]);
			pipe([
				[-grid/2, 0],
				[-grid*0.4, grid*0.1],
				[-grid*0.4, grid*0.3],
			]);
			translate([-grid*0.1, grid*0.2]) cube([grid/4, grid*0.2, emboss_height]);
		}
		cube(grid-tolerance*2, center=true);
	}
}

module plate_1x1_piped_1() {
	plate(1, 1, true);
	
	intersection() {
		translate([0, 0, -emboss_height]) union() {
			pipe([
				[-grid/2, 0],
				[-grid/4, grid*0.1],
				[-grid/4, grid/2],
			]);
			pipe([
				[-grid/4, -grid/2],
				[-grid/4, -grid*0.2],
				[0, -grid*0.1],
				[0, -grid/2],
			]);
			pipe([
				[0, -grid*0.1],
				[grid/4, -grid*0.2],
				[grid/4, -grid/2],
			]);
			pipe([
				[0, grid/2],
				[0, grid*0.1],
				[grid/4, 0],
				[grid/2, 0],
			]);
			pipe([
				[grid/4, grid/2],
				[grid/4, grid*0.4],
			]);
			translate([grid*0.2, grid*0.15]) cube([grid*0.2, grid*0.15, emboss_height]);
		}
		cube(grid-tolerance*2, center=true);
	}
}

module plate_1x1_piped_2() {
	plate(1, 1, true);
	
	intersection() {
		translate([0, 0, -emboss_height]) union() {
			pipe([
				[-grid/4, -grid/2],
				[-grid/4, -grid/4],
				[-grid/2, 0],
			]);
			pipe([
				[0, -grid/2],
				[0, 0],
				[-grid/4, grid/4],
				[-grid/4, grid/2],
			]);
			pipe([
				[0, grid*0.4],
				[0, grid/2],
			]);
			pipe([
				[0, 0],
				[grid/4, grid/4],
				[grid/4, grid/2],
			]);
			pipe([
				[grid/4, -grid/2],
				[grid/4, -grid/4],
			]);
			pipe([
				[grid/2, 0],
				[grid*0.4, 0],
			]);
			translate([grid/4, 0]) sphere(1);
			translate([grid*0.4, grid/4]) sphere(1);
		}
		cube(grid-tolerance*2, center=true);
	}
}

module plate_1x1_piped_3() {
	plate(1, 1, true);
	
	intersection() {
		translate([0, 0, -emboss_height]) union() {
			pipe([
				[-grid/4, -grid/2],
				[-grid/4, -grid/4],
				[-grid*0.15, -grid/4],
			]);
			pipe([
				[-grid/2, 0],
				[grid/2, 0],
			]);
			pipe([
				[-grid/4, grid/2],
				[-grid/4, grid/4],
				[-grid*0.4, grid/4],
			]);
			pipe([
				[0, -grid/2],
				[0, 0],
			]);
			pipe([
				[0, grid/2],
				[0, grid/4],
			]);
			pipe([
				[grid/4, -grid/2],
				[grid/4, -grid/4],
				[grid*0.15, -grid/4],
			]);
			pipe([
				[grid/4, grid/2],
				[grid/4, 0],
			]);
			hull() {
				translate([-grid/4, grid/4, -emboss_height]) sphere(emboss_height*2);
				translate([0, grid/4, -emboss_height]) sphere(emboss_height*2);
			}
		}
		cube(grid-tolerance*2, center=true);
	}
}

module pipe(vertices) {
	radius = emboss_height;
	for (index = [0:len(vertices)-2]) {
		hull() {
			translate(vertices[index]) sphere(radius);
			translate(vertices[index+1]) sphere(radius);
		}
	}
}

plate(1, 1);
translate([grid, 0, 0]) plate(1, 2);
translate([grid*2, 0, 0]) plate(1, 4);

translate([grid*4, 0, 0]) plate(2, 2);
translate([grid*6, 0, 0]) plate(4, 4);

translate([grid*11, 0, 0]) plate_1x1_slatted();
translate([grid*12, 0, 0]) plate_1x1_piped_0();
translate([grid*13, 0, 0]) plate_1x1_piped_1();
translate([grid*14, 0, 0]) plate_1x1_piped_2();
translate([grid*15, 0, 0]) plate_1x1_piped_3();
