include <library.scad>

// Depth of detail.
emboss_height = unit/16;
// Gap around edges where panels will possibly meet.
tolerance = 0.1;

module texture_smooth() {
	overlapping_connector();
	translate([tolerance-unit/2, 0, tolerance-unit/2]) cube([unit-tolerance*2, emboss_height, unit-tolerance*2]);
}

module texture_slatted() {
	slat_count = 3;
	slat_height = (unit-tolerance*2)/slat_count/2;

	overlapping_connector();

	intersection() {
		for (x = [0:slat_count-1]) translate([tolerance-unit/2+slat_height*2*x, -connector_depth, tolerance-unit/2]) cube([slat_height, emboss_height+connector_depth, unit-tolerance*2]);
		safe_volume();
	}
}

module texture_piped_0() {
	overlapping_connector();
	
	intersection() {
		mirror([0, 1, -1]) union() {
			pipe([
				[-grid/4, -grid/2],
				[-grid/4, grid/2],
			]);
			pipe([
				[0, -grid/2],
				[0, -grid*0.1],
				[grid/4, grid*0.1],
				[grid/4, grid/2],
			]);
			pipe([
				[0, grid*0.3],
				[0, grid/2],
			]);
			pipe([
				[grid/4, -grid/2],
				[grid/4, -grid*0.2],
				[grid/2, 0],
			]);
			pipe([
				[-grid/2, 0],
				[-grid*0.4, grid*0.1],
				[-grid*0.4, grid*0.3],
			]);
			translate([-grid*0.1, grid*0.2]) cube([grid/4, grid*0.2, emboss_height]);
		}
		safe_volume();
	}
}

module texture_piped_1() {
	overlapping_connector();

	intersection() {
		mirror([0, 1, -1]) union() {
			pipe([
				[-grid/2, 0],
				[-grid/4, grid*0.1],
				[-grid/4, grid/2],
			]);
			pipe([
				[-grid/4, -grid/2],
				[-grid/4, -grid*0.2],
				[0, -grid*0.1],
				[0, -grid/2],
			]);
			pipe([
				[0, -grid*0.1],
				[grid/4, -grid*0.2],
				[grid/4, -grid/2],
			]);
			pipe([
				[0, grid/2],
				[0, grid*0.1],
				[grid/4, 0],
				[grid/2, 0],
			]);
			pipe([
				[grid/4, grid/2],
				[grid/4, grid*0.4],
			]);
			translate([grid*0.2, grid*0.15]) cube([grid*0.2, grid*0.15, emboss_height]);
		}
		safe_volume();
	}
}

module texture_piped_2() {
	overlapping_connector();

	intersection() {
		mirror([0, 1, -1]) union() {
			pipe([
				[-grid/4, -grid/2],
				[-grid/4, -grid/4],
				[-grid/2, 0],
			]);
			pipe([
				[0, -grid/2],
				[0, 0],
				[-grid/4, grid/4],
				[-grid/4, grid/2],
			]);
			pipe([
				[0, grid*0.4],
				[0, grid/2],
			]);
			pipe([
				[0, 0],
				[grid/4, grid/4],
				[grid/4, grid/2],
			]);
			pipe([
				[grid/4, -grid/2],
				[grid/4, -grid/4],
			]);
			pipe([
				[grid/2, 0],
				[grid*0.4, 0],
			]);
			translate([grid/4, 0]) sphere(1);
			translate([grid*0.4, grid/4]) sphere(1);
		}
		safe_volume();
	}
}

module texture_piped_3() {
	overlapping_connector();

	intersection() {
		mirror([0, 1, -1]) union() {
			pipe([
				[-grid/4, -grid/2],
				[-grid/4, -grid/4],
				[-grid*0.15, -grid/4],
			]);
			pipe([
				[-grid/2, 0],
				[grid/2, 0],
			]);
			pipe([
				[-grid/4, grid/2],
				[-grid/4, grid/4],
				[-grid*0.4, grid/4],
			]);
			pipe([
				[0, -grid/2],
				[0, 0],
			]);
			pipe([
				[0, grid/2],
				[0, grid/4],
			]);
			pipe([
				[grid/4, -grid/2],
				[grid/4, -grid/4],
				[grid*0.15, -grid/4],
			]);
			pipe([
				[grid/4, grid/2],
				[grid/4, 0],
			]);
			hull() {
				translate([-grid/4, grid/4, -emboss_height]) sphere(emboss_height*2);
				translate([0, grid/4, -emboss_height]) sphere(emboss_height*2);
			}
		}
		safe_volume();
	}
}

module texture_porthole() {
	overlapping_connector();
	
	rotate(-90, [1, 0, 0]) rotate_extrude() polygon([
			[grid*0.23, emboss_height],
			[grid*0.3, emboss_height],
			[grid*0.3+emboss_height, 0],
			[grid*0.3+emboss_height, -connector_depth],
			[grid*0.23-emboss_height, -connector_depth],
			[grid*0.23-emboss_height, 0],
	]);
}

module pipe(vertices) {
	radius = emboss_height;
	for (index = [0:len(vertices)-2]) {
		hull() {
			translate(vertices[index]) sphere(radius);
			translate(vertices[index+1]) sphere(radius);
		}
	}
}

module safe_volume() intersection() {
	rotated_width = (unit-tolerance*2)*sin(45);
	rotate(45, [1, 0, 0]) cube([unit+overlap*2, rotated_width, rotated_width], center=true);
	rotate(45) cube([rotated_width, rotated_width, unit+overlap*2], center=true);
	translate([-unit/2, -connector_depth, -unit/2]) cube([unit, unit/2+connector_depth, unit]);
}

// Modified connector that covers entire face of part underneath.
// This is intended to provide an empty surface for decorations.
// Positioned so that decorations start at y=0 and are embossed outward on
// positive y axis.
module overlapping_connector() translate([0, -connector_depth/2, 0]) rotate(180) {
	// Clips in z axis because we can print them cleanly on this axis.
	intersection() {
		union() {
			// Basic shape of clips.
			linear_extrude(unit-tolerance*2, center=true) {
				for (x = [0, 1]) mirror([x, 0]) {
					// Single clip with centre of connector at origin.
					// Polygon points are relative to the corner of a cube.
					// Clockwise from top.
					translate([-unit/2, connector_tolerance+connector_depth/2]) polygon([
						[slot_lip+slot_clip_length, slot_lip+slot_clip_length-connector_tolerance/sin(45)],
						[slot_lip+slot_clip_length*1.5-connector_tolerance/sin(45), slot_lip+slot_clip_length/2],
						[slot_lip+slot_clip_length-connector_tolerance*sin(45), slot_lip+connector_tolerance*sin(45)],
						[slot_lip+slot_clip_length-connector_tolerance*sin(45), -connector_tolerance],
						[unit/2+overlap, -connector_tolerance],
						[unit/2+overlap, -connector_tolerance-connector_depth],
						// [slot_lip+slot_clip_length-slot_clip_width-connector_tolerance*sin(45), -connector_tolerance-connector_depth],
						[tolerance, -connector_tolerance-connector_depth],
						[tolerance, -connector_tolerance],
						[slot_lip+slot_clip_length-slot_clip_width-connector_tolerance*sin(45), -connector_tolerance],
						[slot_lip+slot_clip_length-slot_clip_width-connector_tolerance*sin(45), slot_lip+slot_clip_length-slot_clip_width-connector_tolerance*2],
					]);
				}
			}
			// Rails in x axis because we can't print clips properly for this axis, but we
			// still be the connector to keep itself from slipping.
			intersection() {
				// Basic shape of rails.
				rotate(90, [0, 1, 0]) linear_extrude(unit-(slot_lip+slot_clip_length+0.4)*2, center=true) {
					for (x = [0, 1]) mirror([x, 0]) {
						// Single rail with centre of connector at origin.
						// Polygon points are relative to the corner of a cube.
						// Clockwise from top.
						translate([-unit/2, connector_tolerance+connector_depth/2]) polygon([
							[tolerance, slot_lip/2+slot_clip_width-connector_tolerance],
							[slot_lip+slot_clip_length-slot_clip_width, slot_lip/2+slot_clip_width-connector_tolerance],
							[slot_lip+slot_clip_length+slot_lip/2-connector_tolerance*sin(45), -connector_tolerance],
							[slot_lip+slot_clip_length+slot_lip/2-connector_tolerance*sin(45), -connector_tolerance-connector_depth],
							// [slot_lip+slot_clip_length-slot_clip_width, -connector_tolerance-connector_depth],
							[tolerance, -connector_tolerance-connector_depth],
						]);
					}
				}
				// Cut off corners so they don't join to clips and prevent them flexing.
				rotate(45) cube([(unit/2+slot_clip_width-slot_lip-slot_clip_length)*sin(45)*2, (unit/2+slot_clip_width-slot_lip-slot_clip_length)*sin(45)*2, unit*2], center=true);
			}
		}
		// Cut off corners so that connectors don't interfere.
		safe_volume_side = (unit/2+connector_tolerance+connector_depth/2)/sin(45)-connector_tolerance*2;
		rotate(45, [1, 0, 0]) cube([unit+overlap*2, safe_volume_side, safe_volume_side], center=true);
	}
	// Support.
	translate([tolerance-unit/2, 0, tolerance-unit/2]) cube([1, slot_clip_length+slot_clip_width+connector_depth/2, 2]);
	translate([unit/2-tolerance-1, 0, tolerance-unit/2]) cube([1, slot_clip_length+slot_clip_width+connector_depth/2, 2]);
}

texture_smooth();
translate([grid, 0, 0]) texture_slatted();
translate([grid*2, 0, 0]) texture_piped_0();
translate([grid*3, 0, 0]) texture_piped_1();
translate([grid*4, 0, 0]) texture_piped_2();
translate([grid*5, 0, 0]) texture_piped_3();
translate([grid*6, 0, 0]) texture_porthole();
